# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2, or (at your option)
# any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
# 02111-1307, USA.

# Written by Risto Maki-Petays <risto@kaverit.org>.

# BUGS:
#   Endianness checking is broken
#   Not very safe functions
#   Tags are overriden if multiple tags are read

package LW;

use 5.005;

use strict;
use warnings;
use IO::File;
use File::stat;

our $VERSION = "0.1";

=head1 NAME

LW::Object - A Lightwave object loader

=head1 DESCRIPTION

This is a library which contains some generic functions used by common modules
in the LW subset.

=over

=cut



=item f32 $buffer

Reads a floating point value from a 4-character-string.

=cut

sub f32 {
  my $buf = shift;
  return unpack('f', reverse(split(//, $buf)));
}

=item u32 $buffer

Reads an unsigned 32 bit integer from a 4-character-string.

=cut

sub u32 {
  my $buf = shift;
  my $endian = 'V';
#  if($self->{endian}) { $endian = 'N'; }
  $endian = 'N';
  return unpack($endian, $buf);
}

=item u16 $buffer

Reads an unsigned 16 bit integer from a 4-character-string.

=cut

sub u16 {
  my $buf = shift;
  my $endian = 'v';
#  if($self->{endian}) { $endian = 'n'; }
  $endian = 'n';
  return unpack($endian, $buf);
}

=item plineif $line $var

Prints the specified printf line if $var is true.

B<Example:>

    LW::plineif("Object size: %d bytes\n", $object->{filesize};

=cut

sub plineif {
  my $line = shift;
  my $var = shift;
  if($var) {
    printf($line, $var);
  }
}

1;

