#!/usr/bin/perl -w

# lwo.pl v0.1
# written by Risto Mki-Petays  <basscadet@iki.fi>
# 
# Just a simple Lightwave object loader. Works at least with Lightwave 7.0 and
# 8.0 objects. The code isn't very safe, use at your own risk!

use strict;
use File::stat;
use IO::File;

use SDL;
use SDL::App;
use SDL::Constants;
use SDL::Event;
use SDL::Color;
use SDL::Surface;
use SDL::Cursor;

use Data::Dumper;
use SDL::OpenGL;
use SDL::OpenGL::Cube;
use SDL::OpenGL::Model;
use SDL::OpenGL::Constants;

use SDL::OpenGL;
use SDL::OpenGL::Cube;
use SDL::OpenGL::Model;
use SDL::OpenGL::Constants;

use Time::HiRes;

use LW;
use LW::Object;

our $rot, our $distance = -5.0, our @time;
our %pos;

$pos{'tx'} = 0;
$pos{'ty'} = 0;
$pos{'tz'} = 2;
$pos{'x'} = 0;
$pos{'y'} = 0;
$pos{'z'} = 8;
$time[0] = Time::HiRes::gettimeofday();

my $col = 0;
sub drawgl {
#  my $gl_string = shift;
  my $objdata = shift;
  my @objectdata = @$objdata;
  $time[1] = $time[0];
  $time[0] = Time::HiRes::gettimeofday();
  my $factor = ($time[0] - $time[1]) * 1000;
#  print "$factor\n";

  glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT); # clears the window of any drawing and sets it to the Clear Color in setup
  glLoadIdentity();

#  gluLookAt(100, 90, 150, 100, 90, 149, 0, 1, 0);
#            100, 90, 150, 100.128795596578, 90, 149.008328837617, 0, 1, 0
  gluLookAt($pos{'x'}, $pos{'y'}, $pos{'z'}, $pos{'tx'}, $pos{'ty'}, $pos{'tz'}, 0, 1, 0);

#  glTranslate(-1.5, 0.0, 0);

  $rot->[0]->[0]+=0.010 * $factor;
  $rot->[0]->[1]+=0.020 * $factor;
  $rot->[0]->[2]+=0.030 * $factor;

#  glClearColor(0.30 + $col, 0.00, 0.50, 0.0);
#  $col += 0.001 * $factor; if($col > 0.5) { $col = 0; }

  glRotate($rot->[0]->[0], 1.0, 0.0, 0.0);
  glRotate($rot->[0]->[1], 0.0, 1.0, 0.0);
  glRotate($rot->[0]->[2], 0.0, 0.0, 1.0);

  glBindTexture(GL_TEXTURE_2D, 1);

#  glPushMatrix();
#  glPopMatrix();
  # RGB
  glColor(1.0, 0.0, 0.0); # this function defines the current color to draw in
#  print $gl_string . "\n";
#  eval($gl_string);
  my @gl_type = (0, GL_POINTS, GL_LINES, GL_TRIANGLES, GL_QUADS, GL_POLYGON);
  foreach my $polygon (@objectdata) {
    if(@$polygon) {
      if(@$polygon-1 < 5) {
        glBegin($gl_type[@$polygon-1]);
      } else {
        glBegin($gl_type[5]);
      }
      my $order = 0;
      foreach my $vertex (@$polygon) {
        if($order++ == 0) {
          glNormal(@$vertex[0], @$vertex[1], @$vertex[2]);
        } else {
          glVertex(@$vertex[0], @$vertex[1], @$vertex[2]);
        }
      }
      glEnd();
    }
  }
}
 
sub loadgltex($$) {
  my ($filename, $index) = @_;
  my ($pixels, $width, $height, $size) = imageload($filename);
  glGenTextures(3);
  glBindTexture(GL_TEXTURE_2D, $index);
#  glTexParameter(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_NEAREST);
#  glTexParameter(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST);
  glTexParameter(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
  glTexParameter(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR_MIPMAP_NEAREST);
  glTexImage2D(GL_TEXTURE_2D, 0, 3, $width, $height, 0, GL_RGB, GL_UNSIGNED_BYTE, $pixels);
  gluBuild2DMipmaps(GL_TEXTURE_2D, 3, $width, $height, GL_RGB, GL_UNSIGNED_BYTE, $pixels); 
  my $glerr = glGetError();
  die "Problem setting up 2d Texture (dimensions not a power of 2?)):" . gluErrorString($glerr) . "\n" if $glerr;
}

#something needs to keep the ref count alive for objects which represents data in C space (they have no ref count):
my @ref = ();

sub imageload($) {
  my $filename = shift;
  my $surface = new SDL::Surface( -name => $filename); #makes use of SDL: BMP loader.
  my $width = $surface->width();
  my $height = $surface->height();
  my $bytespp = $surface->bytes_per_pixel();
  my $size = $width * $height * $bytespp;
  my $surface_pixels = $surface->pixels();
  my $surface_size = $width * $height * $surface->bytes_per_pixel();
  my $raw_pixels = $surface_pixels;   
  #do a conversion (the pixel data is accessable as a simple string)
  my $pixels = $raw_pixels;
  my $pre_conv = $pixels;                 #rotate image 180 degrees!
  my $new_pixels = "";
  for (my $y = 0; $y< $height; $y++) {
    my $y_pos = $y * $width * $bytespp;              #calculate offset into the image (a string)
    my $row = substr ($pre_conv, $y_pos, $width * $bytespp); #extract 1 pixel row
    $row =~ s/\G(.)(.)(.)/$3$2$1/gms;                    #turn the BMP BGR order into OpenGL RGB order;  
    $new_pixels .= reverse $row;       
   }
   $raw_pixels  = $new_pixels;                #put transformed data into C array.
   push @ref, $raw_pixels, $surface;
   #we could have created another SDL surface frm the '$raw_pixel's... oh well.
   return ($raw_pixels, $width, $height, $size);
}

sub initgl($$) {
  my ($width, $height) = @_;
  glViewport(0, 0, $width, $height);
#  loadgltex("Data/crate.png", 1);
#  loadgltex("Data/tim.jpg", 2);
#  glEnable(GL_TEXTURE_2D);
  # textures here
  glClearColor(0.30, 0.00, 0.50, 0.0);
  glClearDepth(1.0);
  glDepthFunc(GL_LESS);
  glEnable(GL_DEPTH_TEST);
  glShadeModel(GL_SMOOTH);
#  $camera->setShape(45, $width/$height, 0.1, 500);
  glMatrixMode(GL_PROJECTION);
  glLoadIdentity();

  gluPerspective(45.0, $width/$height, 0.1, 100.0);
  glMatrixMode(GL_MODELVIEW);

   my $LightAmbient  = [ 0.5, 0.5, 0.5, 1.0 ];                   # white ambient light at half intensity (rgba) */

   my $LightDiffuse  = [ 0.0, 1.0, 1.0, 1.0 ];                    # super bright, full intensity diffuse light. */

   my $LightPosition = [ -0.5 , 1.0, 2.0, 1.0 ];                 # position of light (x, y, z, (position of light)) */



   glLight(GL_LIGHT1, GL_AMBIENT, @$LightAmbient);  # add lighting. (ambient)
   glLight(GL_LIGHT1, GL_DIFFUSE, @$LightDiffuse);  # add lighting. (diffuse).
   glLight(GL_LIGHT1, GL_POSITION,@$LightPosition); # set light position.
   glEnable(GL_LIGHT1);                             # turn light 1 on.  
#  glLight(GL_LIGHT1, GL_AMBIENT,  [0.5, 0.5, 0.5, 1.0]);
#  glLight(GL_LIGHT1, GL_DIFFUSE,  [1.0, 1.0, 1.0, 1.0]);
#  glLight(GL_LIGHT1, GL_POSITION, [0.0, 0.0, 2.0, 1.0]);
#  glEnable(GL_LIGHT1);
  glEnable(GL_LIGHTING);
}

my $x = new LW::Object;
if($x->open($ARGV[0])) {
  $x->report;
  #print $x->draw;
#  my $x_gl = $x->draw;
  my @objectdata = $x->getgl;

  my $title = 'My SDL App';
  my ($width, $height, $depth, $fullscreen, $opengl) = ( 1600, 1200, 16, 1, 1);
  #my ($width, $height, $depth, $fullscreen, $opengl) = ( 320, 240, 16, 1, 1);
  my %shapes = ();

  my $vidmode_flags |= SDL_FULLSCREEN if $fullscreen;

  our $app = SDL::App->new(
        -width  => $width,
        -height => $height,
        -depth  => $depth,
        -title  => $title,
        -flags  => $vidmode_flags,
        -opengl => 1,
  );

  SDL::ShowCursor(0);

  ### Main loop
  my $done = 0;
  my $event = new SDL::Event;             # create a new event

  initgl($width, $height);
  # resize($width,$height);
  glEnable(GL_DEPTH_TEST);
  while(!$done) {
    $app->sync();
    $event->pump;
    $event->poll;
    if($event->type == SDL_QUIT) {
      $done = 1;
    } elsif($event->type == SDL_KEYDOWN) {
      my $key = $event->key_sym;
      if($key == SDLK_ESCAPE) {
        $done = 1;
      }
      if($key == SDLK_LEFT) {
        $pos{'x'} -= 0.01;
        $pos{'tx'} -= 0.01;
      }
      if($key == SDLK_RIGHT) {
        $pos{'x'} += 0.01;
        $pos{'tx'} += 0.01;
      }
      if($key == SDLK_UP) {
        $pos{'z'} -= 0.01;
        $pos{'tz'} -= 0.01;
      }
      if($key == SDLK_DOWN) {
        $pos{'z'} += 0.01;
        $pos{'tz'} += 0.01;
      }
      if($key == SDLK_PAGEUP) {
        $pos{'y'} += 0.01;
        $pos{'ty'} += 0.01;
      }
      if($key == SDLK_PAGEDOWN) {
        $pos{'y'} -= 0.01;
        $pos{'ty'} -= 0.01;
      }
    }
#    drawgl($x_gl);
    drawgl([@objectdata]);
  }

}

